@@ -162,6 +162,11 @@ quirk_detailed_use_maximum_size(int scrnIndex, xf86MonPtr DDC)
         DDC->vendor.prod_id == 0x7f01)
         return TRUE;
 
+    /* Sony Vaio Pro 13 */
+    if (memcmp(DDC->vendor.name, "MEI", 4) == 0 &&
+        DDC->vendor.prod_id == 0x96a2)
+        return TRUE;
+
     return FALSE;
 }
 
@@ -708,7 +713,7 @@ static const struct {
     {1600, 1200, 75, 0},
     {1600, 1200, 85, 0},
     {1792, 1344, 60, 0},
-    {1792, 1344, 85, 0},
+    {1792, 1344, 75, 0},
     {1856, 1392, 60, 0},
     {1856, 1392, 75, 0},
     {1920, 1200, 60, 1},
@@ -973,11 +978,11 @@ handle_cea_svd(struct cea_video_block *video, void *data)
 }
 
 static DisplayModePtr
-DDCModesFromCEAExtension(int scrnIndex, xf86MonPtr MonPtr)
+DDCModesFromCEAExtension(int scrnIndex, xf86MonPtr mon_ptr)
 {
     DisplayModePtr Modes = NULL;
 
-    xf86ForEachVideoBlock(MonPtr, handle_cea_svd, &Modes);
+    xf86ForEachVideoBlock(mon_ptr, handle_cea_svd, &Modes);
 
     return Modes;
 }
