@@ -44,6 +44,6 @@ shadowAlloc(int width, int height, int bpp)
 
     /* Cant use PixmapBytePad -- the structure is probably not initialized yet */
     stride = BitmapBytePad(width * bpp);
-    fb = malloc(stride * height);
+    fb = xallocarray(stride, height);
     return fb;
 }
